@@ -128,7 +128,7 @@ pyg_type_from_name (PyObject *self, PyObject *args)
 #endif
     if (!PyArg_ParseTuple(args, "s:gobject.type_from_name", &name))
 	return NULL;
-    type = _pyg_type_from_name(name);
+    type = g_type_from_name(name);
     if (type != 0)
 	return pyg_type_wrapper_new(type);
     repr = PyObject_Repr((PyObject*)self);
@@ -958,7 +958,7 @@ get_type_name_for_class(PyTypeObject *class)
 	for (i = 0; type_name[i] != '\0'; i++)
 	    if (type_name[i] == '.')
 		type_name[i] = '+';
-	if (_pyg_type_from_name(type_name) == 0)
+	if (g_type_from_name(type_name) == 0)
 	    break;              /* we now have a unique name */
 	++name_serial;
     }
@@ -1180,6 +1180,10 @@ pyg_type_register(PyTypeObject *class, const char *type_name)
     g_type_set_qdata(instance_type, g_quark_from_string("PyGObject::class"),
 		     class);
 
+    /* Mark this GType as a custom python type */
+    g_type_set_qdata(instance_type, pygobject_custom_key,
+                     GINT_TO_POINTER (1));
+
     /* set new value of __gtype__ on class */
     gtype = pyg_type_wrapper_new(instance_type);
     PyObject_SetAttrString((PyObject *)class, "__gtype__", gtype);
@@ -1525,31 +1529,13 @@ pygobject_gil_state_release (int flag)
     pyglib_gil_state_release(flag);
 }
 
-static PyObject *
-pyg_threads_init (PyObject *unused, PyObject *args, PyObject *kwargs)
-{
-    if (!pyglib_enable_threads())
-        return NULL;
-
-    Py_INCREF(Py_None);
-    return Py_None;
-}
-
 /* Only for backwards compatibility */
 static int
 pygobject_enable_threads(void)
 {
-    if (!pyglib_enable_threads())
-	return -1;
     return 0;
 }
 
-static void
-pyg_note_threads_enabled(void)
-{
-    pygobject_api_functions.threads_enabled = TRUE;
-}
-
 static PyObject *
 pyg_signal_accumulator_true_handled(PyObject *unused, PyObject *args)
 {
@@ -1690,8 +1676,6 @@ static PyMethodDef _gobject_functions[] = {
       pyg_object_class_list_properties, METH_VARARGS },
     { "new",
       (PyCFunction)pyg_object_new, METH_VARARGS|METH_KEYWORDS },
-    { "threads_init",
-      (PyCFunction)pyg_threads_init, METH_VARARGS|METH_KEYWORDS },
     { "signal_accumulator_true_handled",
       (PyCFunction)pyg_signal_accumulator_true_handled, METH_VARARGS },
     { "add_emission_hook",
@@ -2068,7 +2052,13 @@ struct _PyGObject_Functions pygobject_api_functions = {
   pyg_flags_add,
   pyg_flags_from_gtype,
 
-  FALSE, /* threads_enabled */
+  /* threads_enabled */
+#ifdef DISABLE_THREADING
+  FALSE,
+#else
+  TRUE,
+#endif
+
   pygobject_enable_threads,
   pygobject_gil_state_ensure,
   pygobject_gil_state_release,
@@ -2080,7 +2070,8 @@ struct _PyGObject_Functions pygobject_api_functions = {
   add_warning_redirection,
   disable_warning_redirections,
 
-  pyg_type_register_custom_callback,
+  NULL, /* previously type_register_custom */
+
   pyg_gerror_exception_check,
 
   pyglib_option_group_new,
@@ -2175,9 +2166,6 @@ PYGLIB_MODULE_START(_gobject, "_gobject")
 {
     PyObject *d;
 
-#if !defined(GLIB_VERSION_2_36)
-    g_type_init();
-#endif
     pyglib_init();
 
     d = PyModule_GetDict(module);
@@ -2194,8 +2182,5 @@ PYGLIB_MODULE_START(_gobject, "_gobject")
     pygobject_pointer_register_types(d);
     pygobject_enum_register_types(d);
     pygobject_flags_register_types(d);
-
-    pygobject_api_functions.threads_enabled = pyglib_threads_enabled();
-    _pyglib_notify_on_enabling_threads(pyg_note_threads_enabled);
 }
 PYGLIB_MODULE_END
