@@ -45,6 +45,7 @@ static void pygobject_inherit_slots(PyTypeObject *type, PyObject *bases,
 static void pygobject_find_slot_for(PyTypeObject *type, PyObject *bases, int slot_offset,
 				    gboolean check_for_present);
 GType PY_TYPE_OBJECT = 0;
+GQuark pygobject_custom_key;
 GQuark pygobject_class_key;
 GQuark pygobject_class_init_key;
 GQuark pygobject_wrapper_key;
@@ -149,8 +150,6 @@ pygobject_get_inst_data(PyGObject *self)
 }
 
 
-GHashTable *custom_type_registration = NULL;
-
 PyTypeObject *PyGObject_MetaType = NULL;
 
 /**
@@ -300,15 +299,16 @@ PyGProps_getattro(PyGProps *self, PyObject *attr)
         return pyg_param_spec_new(pspec);
     }
 
-    /* See if the property's class is from the gi repository. If so,
-     * use gi to correctly read the property value. */
-    ret = pygi_get_property_value (self->pygobject, pspec);
-    if (ret != NULL) {
-        return ret;
+    if (!pyg_gtype_is_custom (pspec->owner_type)) {
+        /* The GType is not implemented at the Python level: see if we can
+         * read the property value via gi. */
+        ret = pygi_get_property_value (self->pygobject, pspec);
+        if (ret)
+            return ret;
     }
-    
-    /* If we reach here, it must be a property defined outside of gi.
-     * Just do a straightforward read. */
+
+    /* The GType is implemented in Python, or we failed to read it via gi:
+     * do a straightforward read. */
     g_value_init(&value, G_PARAM_SPEC_VALUE_TYPE(pspec));
     pyg_begin_allow_threads;
     g_object_get_property(self->pygobject->obj, pspec->name, &value);
@@ -403,18 +403,18 @@ PyGProps_setattro(PyGProps *self, PyObject *attr, PyObject *pvalue)
     if (!pspec) {
 	return PyObject_GenericSetAttr((PyObject *)self, attr, pvalue);
     }
-
-    /* See if the property's class is from the gi repository. If so,
-     * use gi to correctly read the property value. */
-    ret = pygi_set_property_value (self->pygobject, pspec, pvalue);
-    if (ret == 0)
-        return 0;
-    else if (ret == -1)
-        if (PyErr_Occurred())
+    if (!pyg_gtype_is_custom (pspec->owner_type)) {
+        /* This GType is not implemented in Python: see if we can set the
+         * property via gi. */
+        ret = pygi_set_property_value (self->pygobject, pspec, pvalue);
+        if (ret == 0)
+            return 0;
+        else if (ret == -1 && PyErr_Occurred())
             return -1;
+    }
 
-    /* If we reach here, it must be a property defined outside of gi.
-     * Just do a straightforward set. */
+    /* This GType is implemented in Python, or we failed to set it via gi:
+     * do a straightforward set. */
     if (!set_property_from_pspec(obj, pspec, pvalue))
 	return -1;
 				  
@@ -934,10 +934,6 @@ pygobject_lookup_class(GType gtype)
     if (gtype == G_TYPE_INTERFACE)
 	return &PyGInterface_Type;
     
-    py_type = pyg_type_get_custom(g_type_name(gtype));
-    if (py_type)
-	return py_type;
-
     py_type = g_type_get_qdata(gtype, pygobject_class_key);
     if (py_type == NULL) {
 	py_type = g_type_get_qdata(gtype, pyginterface_type_key);
@@ -1638,15 +1634,52 @@ pygobject_bind_property(PyGObject *self, PyObject *args)
 	return pygbinding_weak_ref_new(G_OBJECT (binding));
 }
 
+static PyObject *
+connect_helper(PyGObject *self, gchar *name, PyObject *callback, PyObject *extra_args, PyObject *object, gboolean after)
+{
+    guint sigid;
+    GQuark detail = 0;
+    GClosure *closure = NULL;
+    gulong handlerid;
+    GSignalQuery query_info;
+
+    if (!g_signal_parse_name(name, G_OBJECT_TYPE(self->obj),
+			     &sigid, &detail, TRUE)) {
+	PyObject *repr = PyObject_Repr((PyObject*)self);
+	PyErr_Format(PyExc_TypeError, "%s: unknown signal name: %s",
+		     PYGLIB_PyUnicode_AsString(repr),
+		     name);
+	Py_DECREF(repr);
+	return NULL;
+    }
+
+    g_signal_query (sigid, &query_info);
+    if (!pyg_gtype_is_custom (query_info.itype)) {
+        /* The signal is implemented by a non-Python class, probably
+         * something in the gi repository. */
+        closure = pygi_signal_closure_new (self, query_info.itype,
+                                           query_info.signal_name, callback,
+                                           extra_args, object);
+    }
+
+    if (!closure) {
+        /* The signal is either implemented at the Python level, or it comes
+         * from a foreign class that we don't have introspection data for. */
+        closure = pyg_closure_new (callback, extra_args, object);
+    }
+
+    pygobject_watch_closure((PyObject *)self, closure);
+    handlerid = g_signal_connect_closure_by_id(self->obj, sigid, detail,
+					       closure, after);
+    return PyLong_FromUnsignedLong(handlerid);
+}
+
 static PyObject *
 pygobject_connect(PyGObject *self, PyObject *args)
 {
-    PyObject *first, *callback, *extra_args, *repr = NULL;
+    PyObject *first, *callback, *extra_args, *ret;
     gchar *name;
-    guint sigid, len;
-    gulong handlerid;
-    GQuark detail = 0;
-    GClosure *closure;
+    guint len;
 
     len = PyTuple_Size(args);
     if (len < 2) {
@@ -1667,40 +1700,21 @@ pygobject_connect(PyGObject *self, PyObject *args)
     
     CHECK_GOBJECT(self);
     
-    if (!g_signal_parse_name(name, G_OBJECT_TYPE(self->obj),
-			     &sigid, &detail, TRUE)) {
-	repr = PyObject_Repr((PyObject*)self);
-	PyErr_Format(PyExc_TypeError, "%s: unknown signal name: %s",
-		     PYGLIB_PyUnicode_AsString(repr),
-		     name);
-	Py_DECREF(repr);
-	return NULL;
-    }
     extra_args = PySequence_GetSlice(args, 2, len);
     if (extra_args == NULL)
 	return NULL;
 
-    closure = pygi_signal_closure_new(self, name, callback, extra_args, NULL);
-    if (closure == NULL)
-        closure = pyg_closure_new(callback, extra_args, NULL);
-
-    pygobject_watch_closure((PyObject *)self, closure);
-    handlerid = g_signal_connect_closure_by_id(self->obj, sigid, detail,
-					       closure, FALSE);
+    ret = connect_helper(self, name, callback, extra_args, NULL, FALSE);
     Py_DECREF(extra_args);
-    return PyLong_FromUnsignedLong(handlerid);
+    return ret;
 }
 
 static PyObject *
 pygobject_connect_after(PyGObject *self, PyObject *args)
 {
-    PyObject *first, *callback, *extra_args, *repr = NULL;
+    PyObject *first, *callback, *extra_args, *ret;
     gchar *name;
-    guint sigid;
-    gulong handlerid;
     Py_ssize_t len;
-    GQuark detail;
-    GClosure *closure;
 
     len = PyTuple_Size(args);
     if (len < 2) {
@@ -1722,40 +1736,21 @@ pygobject_connect_after(PyGObject *self, PyObject *args)
     
     CHECK_GOBJECT(self);
     
-    if (!g_signal_parse_name(name, G_OBJECT_TYPE(self->obj),
-			     &sigid, &detail, TRUE)) {
-	repr = PyObject_Repr((PyObject*)self);
-	PyErr_Format(PyExc_TypeError, "%s: unknown signal name: %s",
-		     PYGLIB_PyUnicode_AsString(repr),
-		     name);
-	Py_DECREF(repr);
-	return NULL;
-    }
     extra_args = PySequence_GetSlice(args, 2, len);
     if (extra_args == NULL)
 	return NULL;
 
-    closure = pygi_signal_closure_new(self, name, callback, extra_args, NULL);
-    if (closure == NULL)
-        closure = pyg_closure_new(callback, extra_args, NULL);
-
-    pygobject_watch_closure((PyObject *)self, closure);
-    handlerid = g_signal_connect_closure_by_id(self->obj, sigid, detail,
-					       closure, TRUE);
+    ret = connect_helper(self, name, callback, extra_args, NULL, TRUE);
     Py_DECREF(extra_args);
-    return PyLong_FromUnsignedLong(handlerid);
+    return ret;
 }
 
 static PyObject *
 pygobject_connect_object(PyGObject *self, PyObject *args)
 {
-    PyObject *first, *callback, *extra_args, *object, *repr = NULL;
+    PyObject *first, *callback, *extra_args, *object, *ret;
     gchar *name;
-    guint sigid;
-    gulong handlerid;
     Py_ssize_t len;
-    GQuark detail;
-    GClosure *closure;
 
     len = PyTuple_Size(args);
     if (len < 3) {
@@ -1777,40 +1772,21 @@ pygobject_connect_object(PyGObject *self, PyObject *args)
     
     CHECK_GOBJECT(self);
     
-    if (!g_signal_parse_name(name, G_OBJECT_TYPE(self->obj),
-			     &sigid, &detail, TRUE)) {
-	repr = PyObject_Repr((PyObject*)self);
-	PyErr_Format(PyExc_TypeError, "%s: unknown signal name: %s",
-		     PYGLIB_PyUnicode_AsString(repr),
-		     name);
-	Py_DECREF(repr);
-	return NULL;
-    }
     extra_args = PySequence_GetSlice(args, 3, len);
     if (extra_args == NULL)
 	return NULL;
 
-    closure = pygi_signal_closure_new(self, name, callback, extra_args, object);
-    if (closure == NULL)
-        closure = pyg_closure_new(callback, extra_args, object);
-
-    pygobject_watch_closure((PyObject *)self, closure);
-    handlerid = g_signal_connect_closure_by_id(self->obj, sigid, detail,
-					       closure, FALSE);
+    ret = connect_helper(self, name, callback, extra_args, object, FALSE);
     Py_DECREF(extra_args);
-    return PyLong_FromUnsignedLong(handlerid);
+    return ret;
 }
 
 static PyObject *
 pygobject_connect_object_after(PyGObject *self, PyObject *args)
 {
-    PyObject *first, *callback, *extra_args, *object, *repr = NULL;
+    PyObject *first, *callback, *extra_args, *object, *ret;
     gchar *name;
-    guint sigid;
-    gulong handlerid;
     Py_ssize_t len;
-    GQuark detail;
-    GClosure *closure;
 
     len = PyTuple_Size(args);
     if (len < 3) {
@@ -1832,28 +1808,13 @@ pygobject_connect_object_after(PyGObject *self, PyObject *args)
     
     CHECK_GOBJECT(self);
     
-    if (!g_signal_parse_name(name, G_OBJECT_TYPE(self->obj),
-			     &sigid, &detail, TRUE)) {
-	repr = PyObject_Repr((PyObject*)self);
-	PyErr_Format(PyExc_TypeError, "%s: unknown signal name: %s",
-		     PYGLIB_PyUnicode_AsString(repr),
-		     name);
-	Py_DECREF(repr);
-	return NULL;
-    }
     extra_args = PySequence_GetSlice(args, 3, len);
     if (extra_args == NULL)
 	return NULL;
 
-    closure = pygi_signal_closure_new(self, name, callback, extra_args, object);
-    if (closure == NULL)
-        closure = pyg_closure_new(callback, extra_args, object);
-
-    pygobject_watch_closure((PyObject *)self, closure);
-    handlerid = g_signal_connect_closure_by_id(self->obj, sigid, detail,
-					       closure, TRUE);
+    ret = connect_helper(self, name, callback, extra_args, object, TRUE);
     Py_DECREF(extra_args);
-    return PyLong_FromUnsignedLong(handlerid);
+    return ret;
 }
 
 static PyObject *
@@ -2447,6 +2408,7 @@ pygobject_object_register_types(PyObject *d)
 {
     PyObject *o, *descr;
 
+    pygobject_custom_key = g_quark_from_static_string("PyGObject::custom");
     pygobject_class_key = g_quark_from_static_string("PyGObject::class");
     pygobject_class_init_key = g_quark_from_static_string("PyGObject::class-init");
     pygobject_wrapper_key = g_quark_from_static_string("PyGObject::wrapper");
