@@ -44,11 +44,16 @@ GError = _glib.GError
 OptionContext = _glib.OptionContext
 OptionGroup = _glib.OptionGroup
 Pid = _glib.Pid
-PollFD = _glib.PollFD
 spawn_async = _glib.spawn_async
-threads_init = _glib.threads_init
 
-__all__ += ['GError', 'OptionContext', 'OptionGroup', 'Pid', 'PollFD',
+
+def threads_init():
+    warnings.warn('threads_init longer needs to be called. '
+                  'See: https://bugzilla.gnome.org/show_bug.cgi?id=686914',
+                  PyGIDeprecationWarning)
+
+
+__all__ += ['GError', 'OptionContext', 'OptionGroup', 'Pid',
             'spawn_async', 'threads_init']
 
 
@@ -539,6 +544,10 @@ class Source(GLib.Source):
         setattr(source, '__pygi_custom_source', True)
         return source
 
+    def __del__(self):
+        if hasattr(self, '__pygi_custom_source'):
+            self.unref()
+
     # Backwards compatible API for optional arguments
     def attach(self, context=None):
         id = super(Source, self).attach(context)
@@ -785,6 +794,20 @@ IOChannel = override(IOChannel)
 __all__.append('IOChannel')
 
 
+class PollFD(GLib.PollFD):
+    def __new__(cls, fd, events):
+        pollfd = GLib.PollFD()
+        pollfd.__class__ = cls
+        return pollfd
+
+    def __init__(self, fd, events):
+        self.fd = fd
+        self.events = events
+
+PollFD = override(PollFD)
+__all__.append('PollFD')
+
+
 # The real GLib API is child_watch_add(priority, pid, callback, data).
 # The old static bindings had the following API which we still need to support
 # for a while:
