@@ -99,7 +99,6 @@ _pygi_closure_assign_pyobj_to_retval (gpointer retval, PyObject *object,
                 }
 
                 g_base_info_unref (interface_info);
-                break;
             }
         default:
             *(ffi_arg *) retval = (ffi_arg) arg.v_pointer;
@@ -107,19 +106,6 @@ _pygi_closure_assign_pyobj_to_retval (gpointer retval, PyObject *object,
       }
 }
 
-static void
-_pygi_closure_clear_retval (GICallableInfo *callable_info, gpointer retval)
-{
-    GITypeInfo return_type_info;
-    GITypeTag return_type_tag;
-
-    g_callable_info_load_return_type (callable_info, &return_type_info);
-    return_type_tag = g_type_info_get_tag (&return_type_info);
-    if (return_type_tag != GI_TYPE_TAG_VOID) {
-        *((ffi_arg *) retval) = 0;
-    }
-}
-
 static void
 _pygi_closure_assign_pyobj_to_out_argument (gpointer out_arg, PyObject *object,
                                             GITypeInfo *type_info,
@@ -285,7 +271,7 @@ _pygi_closure_convert_ffi_arguments (GICallableInfo *callable_info, void **args)
                         break;
                     } else if (interface_type == GI_INFO_TYPE_ENUM ||
                                interface_type == GI_INFO_TYPE_FLAGS) {
-                        g_args[i].v_uint = * (guint *) args[i];
+                        g_args[i].v_double = * (double *) args[i];
                         g_base_info_unref (interface);
                         break;
                     } else if (interface_type == GI_INFO_TYPE_STRUCT ||
@@ -559,7 +545,6 @@ _pygi_closure_handle (ffi_cif *cif,
     Py_DECREF (py_args);
 
     if (retval == NULL) {
-        _pygi_closure_clear_retval (closure->info, result);
         PyErr_Print();
         goto end;
     }
