@@ -194,19 +194,19 @@ gi_argument_from_c_long (GIArgument *arg_out,
  * expected C union
  */
 static gboolean
-_is_union_member (PyGIInterfaceCache *iface_cache, PyObject *py_arg) {
+_is_union_member (GIInterfaceInfo *interface_info, PyObject *py_arg) {
     gint i;
     gint n_fields;
     GIUnionInfo *union_info;
     GIInfoType info_type;
     gboolean is_member = FALSE;
 
-    info_type = g_base_info_get_type (iface_cache->interface_info);
+    info_type = g_base_info_get_type (interface_info);
 
     if (info_type != GI_INFO_TYPE_UNION)
         return FALSE;
 
-    union_info = (GIUnionInfo *) iface_cache->interface_info;
+    union_info = (GIUnionInfo *) interface_info;
     n_fields = g_union_info_get_n_fields (union_info);
 
     for (i = 0; i < n_fields; i++) {
@@ -696,15 +696,10 @@ check_valid_double (double x, double min, double max)
     return TRUE;
 }
 
-gboolean
-_pygi_marshal_from_py_float (PyGIInvokeState   *state,
-                             PyGICallableCache *callable_cache,
-                             PyGIArgCache      *arg_cache,
-                             PyObject          *py_arg,
-                             GIArgument        *arg)
+static gboolean
+_pygi_py_arg_to_double (PyObject *py_arg, double *double_)
 {
     PyObject *py_float;
-    double double_;
 
     if (!PyNumber_Check (py_arg)) {
         PyErr_Format (PyExc_TypeError, "Must be number, not %s",
@@ -716,14 +711,29 @@ _pygi_marshal_from_py_float (PyGIInvokeState   *state,
     if (!py_float)
         return FALSE;
 
-    double_ = PyFloat_AsDouble (py_float);
+    *double_ = PyFloat_AsDouble (py_float);
     Py_DECREF (py_float);
 
+
+    return TRUE;
+}
+
+gboolean
+_pygi_marshal_from_py_float (PyGIInvokeState   *state,
+                             PyGICallableCache *callable_cache,
+                             PyGIArgCache      *arg_cache,
+                             PyObject          *py_arg,
+                             GIArgument        *arg)
+{
+    double double_;
+
+    if (!_pygi_py_arg_to_double (py_arg, &double_))
+        return FALSE;
+
     if (PyErr_Occurred () || !check_valid_double (double_, -G_MAXFLOAT, G_MAXFLOAT))
         return FALSE;
 
     arg->v_float = double_;
-
     return TRUE;
 }
 
@@ -734,27 +744,15 @@ _pygi_marshal_from_py_double (PyGIInvokeState   *state,
                               PyObject          *py_arg,
                               GIArgument        *arg)
 {
-    PyObject *py_float;
     double double_;
 
-    if (!PyNumber_Check (py_arg)) {
-        PyErr_Format (PyExc_TypeError, "Must be number, not %s",
-                      py_arg->ob_type->tp_name);
+    if (!_pygi_py_arg_to_double (py_arg, &double_))
         return FALSE;
-    }
-
-    py_float = PyNumber_Float (py_arg);
-    if (!py_float)
-        return FALSE;
-
-    double_ = PyFloat_AsDouble (py_float);
-    Py_DECREF (py_float);
 
     if (PyErr_Occurred () || !check_valid_double (double_, -G_MAXDOUBLE, G_MAXDOUBLE))
         return FALSE;
 
     arg->v_double = double_;
-
     return TRUE;
 }
 
@@ -768,11 +766,19 @@ _pygi_marshal_from_py_unichar (PyGIInvokeState   *state,
     Py_ssize_t size;
     gchar *string_;
 
+    if (py_arg == Py_None) {
+        arg->v_uint32 = 0;
+        return FALSE;
+    }
+
     if (PyUnicode_Check (py_arg)) {
        PyObject *py_bytes;
 
        size = PyUnicode_GET_SIZE (py_arg);
        py_bytes = PyUnicode_AsUTF8String (py_arg);
+       if (!py_bytes)
+           return FALSE;
+
        string_ = g_strdup(PYGLIB_PyBytes_AsString (py_bytes));
        Py_DECREF (py_bytes);
 
@@ -804,6 +810,7 @@ _pygi_marshal_from_py_unichar (PyGIInvokeState   *state,
 
     return TRUE;
 }
+
 gboolean
 _pygi_marshal_from_py_gtype (PyGIInvokeState   *state,
                              PyGICallableCache *callable_cache,
@@ -1636,65 +1643,16 @@ _pygi_marshal_from_py_interface_struct (PyGIInvokeState   *state,
 {
     PyGIInterfaceCache *iface_cache = (PyGIInterfaceCache *)arg_cache;
 
-    if (py_arg == Py_None) {
-        arg->v_pointer = NULL;
-        return TRUE;
-    }
-
-    /* FIXME: handle this large if statement in the cache
-     *        and set the correct marshaller
-     */
-
-    if (iface_cache->g_type == G_TYPE_CLOSURE) {
-        return pygi_marshal_from_py_gclosure (py_arg, arg);
-    } else if (iface_cache->g_type == G_TYPE_VALUE) {
-        return pygi_marshal_from_py_gvalue(py_arg, arg,
-                                           arg_cache->transfer,
-                                           TRUE);
-    } else if (iface_cache->is_foreign) {
-        PyObject *success;
-        success = pygi_struct_foreign_convert_to_g_argument (py_arg,
-                                                             iface_cache->interface_info,
-                                                             arg_cache->transfer,
-                                                             arg);
-
-        return (success == Py_None);
-    } else if (!PyObject_IsInstance (py_arg, iface_cache->py_type)) {
-        /* first check to see if this is a member of the expected union */
-        if (!_is_union_member (iface_cache, py_arg)) {
-            if (!PyErr_Occurred()) {
-                PyObject *module = PyObject_GetAttrString(py_arg, "__module__");
-
-                PyErr_Format (PyExc_TypeError, "argument %s: Expected %s, but got %s%s%s",
-                              arg_cache->arg_name ? arg_cache->arg_name : "self",
-                              iface_cache->type_name,
-                              module ? PYGLIB_PyUnicode_AsString(module) : "",
-                              module ? "." : "",
-                              py_arg->ob_type->tp_name);
-                if (module)
-                    Py_DECREF (module);
-            }
-
-            return FALSE;
-        }
-    }
-
-    if (g_type_is_a (iface_cache->g_type, G_TYPE_BOXED)) {
-        arg->v_pointer = pyg_boxed_get (py_arg, void);
-        if (arg_cache->transfer == GI_TRANSFER_EVERYTHING) {
-            arg->v_pointer = g_boxed_copy (iface_cache->g_type, arg->v_pointer);
-        }
-    } else if (g_type_is_a (iface_cache->g_type, G_TYPE_POINTER) ||
-                   g_type_is_a (iface_cache->g_type, G_TYPE_VARIANT) ||
-                       iface_cache->g_type  == G_TYPE_NONE) {
-        arg->v_pointer = pyg_pointer_get (py_arg, void);
-    } else {
-        PyErr_Format (PyExc_NotImplementedError,
-                      "structure type '%s' is not supported yet",
-                      g_type_name(iface_cache->g_type));
-        return FALSE;
-    }
-    return TRUE;
+    return pygi_marshal_from_py_interface_struct (py_arg,
+                                                  arg,
+                                                  arg_cache->arg_name,
+                                                  iface_cache->interface_info,
+                                                  arg_cache->type_info,
+                                                  iface_cache->g_type,
+                                                  iface_cache->py_type,
+                                                  arg_cache->transfer,
+                                                  arg_cache->is_caller_allocates,
+                                                  iface_cache->is_foreign);
 }
 
 gboolean
@@ -1768,7 +1726,7 @@ gboolean _pygi_marshal_from_py_interface_instance (PyGIInvokeState   *state,
 
             if (!PyObject_IsInstance (py_arg, iface_cache->py_type)) {
                 /* wait, we might be a member of a union so manually check */
-                if (!_is_union_member (iface_cache, py_arg)) {
+                if (!_is_union_member (iface_cache->interface_info, py_arg)) {
                     if (!PyErr_Occurred()) {
                         PyObject *module = PyObject_GetAttrString(py_arg, "__module__");
                         PyErr_Format (PyExc_TypeError,
@@ -1898,14 +1856,13 @@ pygi_marshal_from_py_gobject (PyObject *py_arg, /*in*/
  * py_arg: (in):
  * arg: (out):
  * transfer:
- * copy_reference: TRUE if arg should use the pointer reference held by py_arg
- *                 when it is already holding a GValue vs. copying the value.
+ * is_allocated: TRUE if arg->v_pointer is an already allocated GValue
  */
 gboolean
 pygi_marshal_from_py_gvalue (PyObject *py_arg,
                              GIArgument *arg,
                              GITransfer transfer,
-                             gboolean copy_reference) {
+                             gboolean is_allocated) {
     GValue *value;
     GType object_type;
 
@@ -1915,21 +1872,24 @@ pygi_marshal_from_py_gvalue (PyObject *py_arg,
         return FALSE;
     }
 
+    if (is_allocated)
+        value = (GValue *)arg->v_pointer;
+    else
+        value = g_slice_new0 (GValue);
+
     /* if already a gvalue, use that, else marshal into gvalue */
     if (object_type == G_TYPE_VALUE) {
         GValue *source_value = pyg_boxed_get (py_arg, GValue);
-        if (copy_reference) {
-            value = source_value;
-        } else {
-            value = g_slice_new0 (GValue);
+        if (G_VALUE_TYPE (value) == G_TYPE_INVALID)
             g_value_init (value, G_VALUE_TYPE (source_value));
-            g_value_copy (source_value, value);
-        }
+        g_value_copy (source_value, value);
     } else {
-        value = g_slice_new0 (GValue);
-        g_value_init (value, object_type);
+        if (G_VALUE_TYPE (value) == G_TYPE_INVALID)
+            g_value_init (value, object_type);
+
         if (pyg_value_from_pyobject (value, py_arg) < 0) {
-            g_slice_free (GValue, value);
+            if (!is_allocated)
+                g_slice_free (GValue, value);
             PyErr_SetString (PyExc_RuntimeError, "PyObject conversion to GValue failed");
             return FALSE;
         }
@@ -1970,3 +1930,93 @@ pygi_marshal_from_py_gclosure(PyObject *py_arg,
     arg->v_pointer = closure;
     return TRUE;
 }
+
+gboolean
+pygi_marshal_from_py_interface_struct (PyObject *py_arg,
+                                       GIArgument *arg,
+                                       const gchar *arg_name,
+                                       GIBaseInfo *interface_info,
+                                       GITypeInfo *type_info,
+                                       GType g_type,
+                                       PyObject *py_type,
+                                       GITransfer transfer,
+                                       gboolean is_allocated,
+                                       gboolean is_foreign)
+{
+    if (py_arg == Py_None) {
+        arg->v_pointer = NULL;
+        return TRUE;
+    }
+
+    /* FIXME: handle this large if statement in the cache
+     *        and set the correct marshaller
+     */
+
+    if (g_type_is_a (g_type, G_TYPE_CLOSURE)) {
+        return pygi_marshal_from_py_gclosure (py_arg, arg);
+    } else if (g_type_is_a (g_type, G_TYPE_VALUE)) {
+        return pygi_marshal_from_py_gvalue(py_arg,
+                                           arg,
+                                           transfer,
+                                           is_allocated);
+    } else if (is_foreign) {
+        PyObject *success;
+        success = pygi_struct_foreign_convert_to_g_argument (py_arg,
+                                                             interface_info,
+                                                             transfer,
+                                                             arg);
+
+        return (success == Py_None);
+    } else if (!PyObject_IsInstance (py_arg, py_type)) {
+        /* first check to see if this is a member of the expected union */
+        if (!_is_union_member (interface_info, py_arg)) {
+            if (!PyErr_Occurred()) {
+                gchar *type_name = _pygi_g_base_info_get_fullname (interface_info);
+                PyObject *module = PyObject_GetAttrString(py_arg, "__module__");
+
+                PyErr_Format (PyExc_TypeError, "argument %s: Expected %s, but got %s%s%s",
+                              arg_name ? arg_name : "self",
+                              type_name,
+                              module ? PYGLIB_PyUnicode_AsString(module) : "",
+                              module ? "." : "",
+                              py_arg->ob_type->tp_name);
+                if (module)
+                    Py_DECREF (module);
+                g_free (type_name);
+            }
+
+            return FALSE;
+        }
+    }
+
+    if (g_type_is_a (g_type, G_TYPE_BOXED)) {
+        if (pyg_boxed_check (py_arg, g_type)) {
+            arg->v_pointer = pyg_boxed_get (py_arg, void);
+            if (transfer == GI_TRANSFER_EVERYTHING) {
+                arg->v_pointer = g_boxed_copy (g_type, arg->v_pointer);
+            }
+        } else {
+            PyErr_Format (PyExc_TypeError, "wrong boxed type");
+            return FALSE;
+        }
+
+    } else if (g_type_is_a (g_type, G_TYPE_POINTER) ||
+               g_type_is_a (g_type, G_TYPE_VARIANT) ||
+               g_type  == G_TYPE_NONE) {
+        g_warn_if_fail (g_type_is_a (g_type, G_TYPE_VARIANT) || !g_type_info_is_pointer (type_info) || transfer == GI_TRANSFER_NOTHING);
+
+        if (g_type_is_a (g_type, G_TYPE_VARIANT) &&
+                pyg_type_from_object (py_arg) != G_TYPE_VARIANT) {
+            PyErr_SetString (PyExc_TypeError, "expected GLib.Variant");
+            return FALSE;
+        }
+        arg->v_pointer = pyg_pointer_get (py_arg, void);
+
+    } else {
+        PyErr_Format (PyExc_NotImplementedError,
+                      "structure type '%s' is not supported yet",
+                      g_type_name(g_type));
+        return FALSE;
+    }
+    return TRUE;
+}
